/* Changes recomb. fracts. into centi-Morgans Haldane  */
double cmH (double xx)
{
   double a;
   a = -50 * log (1 - 2 * xx);
   return (a);
}

/* Changes genetic distance in Haldane centi-Morgans to RFs*/
double rf (double yy)
{
   double b;
   b = .5 * (1 - exp (-yy / 50));
   return (b);
}

/* Generates initial parameter estiamtes   */
void get_init (int sxs_flg, double **mapa, double **mapb,
               double *rate, double alpha)
{
   double u, x, ii = 0, F = 0;          /* rate ~ Beta(2,k) if error model is used */
   int   i, j, k = 75;

   u = rans ();
   if (MapEstMod)
   {
      while (F < u)
      {
         ii++;
         x = ii / 1000;
         F = (1 - pow ((1 - x), (k + 1))) - ((k + 1) * pow ((1 - x), k) * x);
      }
      *rate = x - .0005;                /* continuity correction */
   }

   for (j = 1; j < NMarks; j++)         /* Add random error to given map */
   {
      if (sxs_flg)
      {
         for (j = 1; j < NMarks; j++)
            for (i = 0; i < 2; i++)
               mapb[i][j] = alpha * mapa[i][j] + (1 - alpha) * u * 0.5;
      }
      else
      {
         mapb[0][j] = rf (0.5 * (cmH (mapa[0][j]) + cmH (mapa[1][j])));
         mapb[0][j] = alpha * mapb[0][j] + (1 - alpha) * u * 0.5;
         mapb[1][j] = mapb[0][j];
      }
   }
}

/* Copy S.j in to S.j^  */
void swcpy (int *a, int *b, int *c, int *d)
{
   int   i;
   for (i = 0; i < pedsize; i++)
   {
      c[i] = a[i];
      d[i] = b[i];
   }
}

/* Compute and evaluate the Hastings Ratio */
void decide (double *x)
{
   double u;
   x[0] = exp (x[1] + x[2] - x[3] - x[4]);
   u = rans ();
#if 0
   int   i;
   for (i = 1; i <= 4; i++)
      printf (" x[%d] = %6.4f \n", i, x[i]);
   printf (" u is: %6.4f, logprob: %6.4f, hratio is: %6.4f \n",
           u, (x[1] + x[2] - x[3] - x[4]), x[0]);
   printf ("\n runif = %6.6f, hratio = %6.6f, ", u, x[0]);
#endif
   if (u < x[0])
      x[0] = 1;
   else
      x[0] = 0;
/*  printf(" accept = %1.0f \n", x[0]); */
}

/* Prints a map estimation section header to the screen */
void pr_section (int type, int run)
{
   int   i;
   char  sxa[13] = "sex-averaged", sxs[13] = "sex-specific";
   char *name;

   run++;
   if (type == 1)
      name = sxa;
   else
      name = sxs;
   printf ("\n");
   for (i = 1; i <= 55; i++)
      printf ("=");
   printf ("\n Run %d: The %s recombination fractions (RFs) ", run, name);
   printf ("\n        used to initialize the hybrid algorithm: \n");
   for (i = 1; i <= 55; i++)
      printf ("=");
   printf ("\n");
}

/* Determines the increment between source and destination maps to move to.  */
double find_intv (double xx, double yy, int *z)
{
   double a, b, c;
   double b_1 = 0.012, b_2 = 0.905, b_3 = 1.1905;

   a = cmH (xx);
   b = cmH (yy);

   if (b > a)                           /* if the (dn) interval > (se) interval,
                                         * then expand (se)   */
   {
      c = b_1 * a * a + b_2 * a + b_3;
      if (c > b)
         c = b;
   }
   else                                 /* if (dn) interval <= (se) interval,
                                         * then shrink (se)   */
   {
      c = (sqrt (b_2 * b_2 - 4 * b_1 * (b_3 - a)) - b_2) / (2 * b_1);
      if (c < b)
         c = b;
   }
   if (c != b)
      (*z) = 1;                         /* (i.e. continue to accumulate)    */
   c = rf (c);
   return (c);
}

/* Copies mapa to mapb and resets the marker map */
/* Performs burn-in if burn = 1.                 */
void reset_map (int burn, int nsij,
                double **mapa, double **mapb, double **probs)
{
   int   sex, i, a = 0;
   double z = 0.5;

   cpy_map (mapa, mapb);
   for (sex = FEM; sex <= MAL; sex++)
      set_chromosomes (NMarks, mapb[sex], 0, &a, &z, sex);
   set_positions (NMarks);
   if (burn)
      for (i = 1; i <= NBrnIt; i++)
         sampling (0, 1, NMarks, nsij, nFGL, probs);
}

/* Checks to see if two maps are 'close'   */
int comp_map (double z, double **map_a, double **map_b)
{
   int   i, j, k = 1;
   double temp;

   for (i = 0; i < 2; i++)
      for (j = 1; j < NMarks; j++)
      {
         temp = fabs (map_a[i][j] - map_b[i][j]);
         if (temp > z)
            k = 0;
      }
   return (k);
}

/* Puts the contents of map_a into map_b   */
void cpy_map (double **map_a, double **map_b)
{
   int   i, j;

   for (i = 0; i < 2; i++)
      for (j = 1; j < NMarks; j++)
         map_b[i][j] = map_a[i][j];

}

/* Prints a map to the screen   */
void pr_map (int sxs_flg, double **map)
{
   int   j;

   if (sxs_flg == 0)
   {                                    /* Then print a sex-averaged map.  */
      printf ("\n Interval   Sex-Averaged (RF)");
      printf ("\n --------   -----------------\n");
      for (j = 1; j < NMarks; j++)
         printf (" %4d  %15.4f \n", j, map[0][j]);
   }
   else
   {
      printf ("\n Interval   Female (RF)   Male (RF)");
      printf ("\n --------   -----------   ---------\n");
      for (j = 1; j < NMarks; j++)
         printf ("%4d %15.4f %11.4f\n", j, map[0][j], map[1][j]);
   }
}

/* Initializes the recomb counts to zero and sums over components  */
void set_rc (int locus, int ***rcounts, double **rcs)
{
   int   c, i, k, n_comps;

   n_comps = n_compnts;
   for (k = 0; k < 3; k++)
      for (i = 0; i < 2; i++)
         rcs[i][k] = 0;

   for (c = 0; c < n_comps; c++)
   {
      rcs[1][2] += rcounts[c][locus][0]; /* totals      */
      rcs[1][0] += rcounts[c][locus][1]; /* recombs     */
      rcs[1][1] += rcounts[c][locus][2]; /* non-recombs */
      rcs[0][2] += rcounts[c][locus][3];
      rcs[0][0] += rcounts[c][locus][4];
      rcs[0][1] += rcounts[c][locus][5];
   }
}
