/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/**
 * @file   globs.h
 *
 * @brief header file defining global variables.
 *
 *   -  global in scope
 *   -  included by main programs and function libraries
 *   -  contains modelling or simulation parameters
 *      derived from the input statements
 *   -  values are stored by Pars/ functions
 *
 * $Id: globs.h,v 1.83.2.30 2011-01-27 21:00:06 slewis Exp $
 **/

#ifndef  GLOBS_H
#define  GLOBS_H

/**
 * @addtogroup Libraries
 * @{
 */

#ifdef  DEFINE_DATA
GLOBAL char * ProgNam = PNAM;           /* name of Morgan program */
#else
GLOBAL char * ProgNam;
#endif  /*DEFINE_DATA*/

/***  scalar variables, etc, derived from the parameter statements  ****/

GLOBAL int    PedSize;
GLOBAL int    NNams,        NInts,        NReals,       NFix,         NNInts;
GLOBAL int    NStdIn,       StdIn[5],     NStdOut,      StdOut[5];
GLOBAL int    NLoci,        NMarks,       NTraits,      NTLocs,
              NLnkdTL,      NUnlnTL;

#ifdef  DEFINE_DATA                                  /* default values */
GLOBAL int SexIn  = UNDEF;
GLOBAL int SexOut = UNDEF;
GLOBAL int SexOp  = UNDEF;
GLOBAL int ObsIn  = UNDEF;
GLOBAL int ObsOp  = UNDEF;
GLOBAL int ObsOut = UNDEF;
GLOBAL int LabelsOut = UNDEF;
GLOBAL int LatentOut = UNDEF;
GLOBAL int DiscardReals = UNDEF;
GLOBAL int MarkSeedSet = N;
GLOBAL int TraitSeedSet = N;
GLOBAL int SamplSeedSet = N;
GLOBAL int PermSeedSet = N;
GLOBAL int MarkSeedsOut = N;
GLOBAL int TraitSeedsOut = N;
GLOBAL int SamplSeedsOut = N;
GLOBAL int PermSeedsOut = N;
GLOBAL int ChronOut = N;
GLOBAL int AllVarOut = N;
GLOBAL int PrintLevel = 0;
GLOBAL int NVary = 0;
#else
GLOBAL int SexIn;
GLOBAL int SexOut;
GLOBAL int SexOp;
GLOBAL int ObsIn;
GLOBAL int ObsOp;
GLOBAL int ObsOut;
GLOBAL int LabelsOut;
GLOBAL int LatentOut;
GLOBAL int DiscardReals;
GLOBAL int MarkSeedSet;
GLOBAL int TraitSeedSet;
GLOBAL int SamplSeedSet;
GLOBAL int PermSeedSet;
GLOBAL int MarkSeedsOut;
GLOBAL int TraitSeedsOut;
GLOBAL int SamplSeedsOut;
GLOBAL int PermSeedsOut;
GLOBAL int ChronOut;
GLOBAL int AllVarOut;
GLOBAL int PrintLevel;
GLOBAL int NVary;
#endif  /*DEFINE_DATA*/

GLOBAL uint32 MarkSeedC,    MarkSeedT,    TraitSeedC,   TraitSeedT;
GLOBAL uint32 SamplSeedC,   SamplSeedT,   PermSeedC,    PermSeedT;

       enum sEnums {NAM, PA, MA, SEX, OBS};

/*****     the following variables are used by Nghds/ functions     *****/

GLOBAL int NColData;                    /*No. of integers + 1 (including*/
                                        /*names, to selectively read    */
                                        /*from ped file                 */
                                        /*Is 1 greater than max SelInt[]*/
                                        /*which doesn't make sense??    */

GLOBAL int MaxNCmps;                    /*No. of components allocated   */

GLOBAL int * SelInt;                    /*Selection/nonselection of     */
                                        /*integers in the  ped file     */
                                        /*The subscript corresponds to  */
                                        /*the integer in the ped record */
                                        /*(starting from 0),            */
                                        /*the value to the column in    */
                                        /*data[][] (starting from 1)    */

GLOBAL int * SelReal;                   /*Selection/nonselection of     */
                                        /*reals (traits) from ped file  */

/**** the following variables and tables are used by Quant functions ****/

GLOBAL int FENo[FIXEFF];                /*For ea fixed effect selected  */
                                        /*the fixed effect no.          */

GLOBAL int TrNo[MAXQT];                /*For traits selected, the      */
                                        /*trait no.'s                   */

GLOBAL int FESex;                       /*Gender a fixed effect?        */

GLOBAL int FfDex[MAXQT][FIXEFF];       /*For ea trait selected, the    */
                                        /*presence/absence of ea fixed  */
                                        /*effects selected              */

GLOBAL int IFix[FIXEFF];                /*Number of levels (categories) */
                                        /*for ea fixed effect selected  */

GLOBAL int IOCor;                       /*flag for printing correlations*/
                                        /*(vs. covariances)             */

GLOBAL int ChkEigC;                     /*flag for printing progress in */
                                        /*computing eigenvalues         */


/*** the following variables are used by Quant/ and PolyEM functions ****/

GLOBAL int FinPheno;                    /*flag for printing final       */
                                        /*adjusted phenotypes           */

GLOBAL double Mufti[MAX_FEP][FIXEFF][LEVELS+1];  /*Fixed effect params; */
                                        /*Up to MAX_FEP combinations    */

GLOBAL double SigAdd, SigErr;           /* variance parameters          */
                                        /* (univariate)                 */

GLOBAL double TauSq[MAXQT*(MAXQT+1)/2],           /* covariances      */
              SigSq[MAXQT*(MAXQT+1)/2];

GLOBAL int MuftiStart[MAX_FEP][FIXEFF]; /* flags for starting values    */
                                        /* given for Mufti[][]          */

GLOBAL double * peigen;

#define IS_UNIVAR      (!strcmp (ProgNam, "univar"))
#define IS_UNIBIG      (!strcmp (ProgNam, "unibig"))
#define IS_BIVAR       (!strcmp (ProgNam, "bivar"))
#define IS_MULTIVAR    (!strcmp (ProgNam, "multivar"))
#define IS_PEDCHECK    (!strcmp (ProgNam, "pedcheck"))
#define IS_KIN         (!strcmp (ProgNam, "kin"))
#define IS_GENEDROP    (!strcmp (ProgNam, "genedrop"))
#define IS_IBDDROP     (!strcmp (ProgNam, "ibddrop"))
#define IS_MARKERDROP  (!strcmp (ProgNam, "markerdrop"))
#define IS_LM_MAP      (!strcmp (ProgNam, "lm_map"))
#define IS_LM_PVAL     (!strcmp (ProgNam, "lm_pval"))
#define IS_LM_IBDTESTS (!strcmp (ProgNam, "lm_ibdtests"))
#define IS_LM_AUTO     (!strcmp (ProgNam, "lm_auto"))
#define IS_LM_LODS     (!strcmp (ProgNam, "lm_lods"))
#define IS_LM_BAYES    (!strcmp (ProgNam, "lm_bayes"))
#define IS_LM_MARKERS  (!strcmp (ProgNam, "lm_markers"))
#define IS_LM_SCHNELL  (!strcmp (ProgNam, "lm_schnell"))
#define IS_LM_TWOQTL   (!strcmp (ProgNam, "lm_twoqtl"))

#define PAR      "par"
#define PED      "ped"
#define MDAT     "mark"
#define IGEN     "eig"
#define PEEL     "peel"
#define SEED     "seed"
#define OPED     "oped"
#define RPED     "rped"
#define OSCORE   "oscor"
#define ORBEST   "orbest"
#define OSEED    "oseed"
#define RSEED    "rseed"
#define XTRA     "xtra"
#define OXTR     "oxtr"
#define RXTR     "rxtr"
#define IRESC    "iresc"
#define ORESC    "oresc"


/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

/**
 * Command Line or Parameter Statement Specified File Type Structore
 */

/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

/**
 * Command Line or Parameter Statement Specified File Type Structore
 */
struct _filtab
{
   char *type;                      /**< file type (com. line keyword)      */
   int command;                     /**< flag for named in command line     */
   int param;                       /**< flag for named in par statemnt     */
   int done;                        /**< flag for file has been read        */
   int parstat;                     /**< flag for contains param stmnts     */
   int reclo;                       /**< flag for rewrite command line type */
   int rewrit;                      /**< flag for rewrite option set        */
   char *pcods;                     /**< progs which use this file type     */
   char *fop;                       /**< fopen mode "r", "w" or "a"         */
   char *fname;                     /**< file name                          */
   char *fdesc;                     /**< description of file                */
};
typedef struct _filtab * FILTAB_PTR;

#ifdef  DEFINE_DATA
 struct _filtab Filtab[] = {
 {PAR   ,N,N,N,Y,N,N,"ugbmpkdirqvtalyeswx","r",NULL,"parameter file"      },
 {PED   ,N,N,N,Y,N,N,"ugbmpkdirqvtalyeswx","r",NULL,"input pedigree file" },
 {MDAT  ,N,N,N,Y,N,N,"         qvtalyeswx","r",NULL,"marker data file"    },
 {IGEN  ,N,N,N,N,N,N,"  bgm             x","r",NULL,"eigenvalue file"     },
 {PEEL  ,N,N,N,N,N,N," u       q        x","r",NULL,"peeling sequence file"},
 {SEED  ,N,N,N,Y,N,N,"      dirqvtalyeswx","r",NULL,"seed file"           },
 {XTRA  ,N,N,N,N,N,N,"ugbmpkdirqvtalyeswx","r",NULL,"input extra file"    },
 {OPED  ,N,N,N,N,N,N,"ugbmpkdirqvtalyeswx","w",NULL,"output pedigree file"},
 {RPED  ,N,N,N,N,Y,N,"ugbmpkdirqvtalyeswx","w",NULL,"output (overwrite) pedigree file"},
 {OSCORE,N,N,N,N,N,N,"            a  e  x","a",NULL,"output score file"   },
 {ORBEST,N,N,N,N,N,N,"              y   x","w",NULL,"output rao-black estimates file"},
 {OSEED ,N,N,N,N,N,N,"      dirqvtalyeswx","a",NULL,"output seed file"    },
 {RSEED ,N,N,N,N,Y,N,"      dirqvtalyeswx","w",NULL,"output (overwrite) seed file"},
 {OXTR  ,N,N,N,N,N,N,"ugbmpkdirqvtalyeswx","a",NULL,"output extra file"   },
 {RXTR  ,N,N,N,N,N,N,"ugbmpkdirqvtalyeswx","w",NULL,"output (overwrite) extra file"},
 {IRESC ,N,N,N,N,N,N,"            al    x","r",NULL,"input rescue file"   },
 {ORESC ,N,N,N,N,N,N,"            al    x","w",NULL,"output rescue file"  },
 {""    ,N,N,N,N,N,N,""                   ,"" ,NULL,""                    }};
#else
  extern struct _filtab Filtab[];
#endif  /*DEFINE_DATA*/

/** @} */

/**** the following variables and arrays are used by PedComp programs ***/
/**
 * @addtogroup PedComp
 * @{
 */
/** compute kinship                                                     */
#define KINSHP  0
/** compute inbreeding                                                  */
#define INBRD  1
/** compute 2-locus inbreeding                                          */
#define INBRD2  2

/** number of components for which kinship computations are requested   */
GLOBAL int NKinCmp;
/**
 * the no. of names of probands for each type of computation and for each
 * component
 */
GLOBAL int ** NKinNams;
/**
 * the names for each computation, for each component and for each proband.
 */
GLOBAL char **** KinNam;
/**
 * the index for each computation, for each component and for each proband.
 */
GLOBAL int *** KinIx;
/**
 * number of recomb fracts.
 */
GLOBAL int NInbreedRF;
/**
 * recomb fracts for computing 2-locus inbreeding coeffs.
 */
GLOBAL double * InbreedRF;
/** @} */
/**** the following variables and tables are used by PolyEM programs ****/
/**
 * @addtogroup polyem
 * @{
 */
                                        /*See limits.h for default      */
                                        /*values                        */
#ifdef  DEFINE_DATA
GLOBAL int ItMo = ITMO;                 /*Breeding-value iterations     */
GLOBAL int ItEM = ITEM;                 /*Number of EM iterations       */
GLOBAL int ItWt = ITWT;                 /*How often to write out estmts */
GLOBAL int IPtEM = 0;                   /*iem =1 partitioned EM         */
                                        /*iem =0 full EM                */
GLOBAL int IEn = 0;                     /*ien =1 fit environmental model*/
GLOBAL double Diff = DIFF;              /*Convergence criterion for     */
                                        /*breeding values iteration     */
GLOBAL int ICh = 0;                     /*ich =1 -- call bigcheig       */
                                        /*to compute eigenvalues        */
GLOBAL int ChkEig = N;                  /*flag for printing eigenvals   */
GLOBAL int ChkGnv = N;                  /*flag for printing G-iverse    */
GLOBAL int ChkGmt = N;                  /*flag for printing G-matrix    */
GLOBAL int ChkTrc = N;                  /*flag for printing trace       */

#else
GLOBAL int ItMo;
GLOBAL int ItEM;
GLOBAL int ItWt;
GLOBAL int IPtEM;
GLOBAL int IEn;
GLOBAL double Diff;
GLOBAL int ICh;
GLOBAL int ChkEig;
GLOBAL int ChkGnv;
GLOBAL int ChkGmt;
GLOBAL int ChkTrc;

#endif  /*DEFINE_DATA*/


GLOBAL int    TauModel[MAXQT*(MAXQT+1)/2],             /* fit/no fit   */
              SigModel[MAXQT*(MAXQT+1)/2];             /* covariances  */
                                                        /* (multivar)   */
/** @} */

/******     the following structures are used by Genedrop          ******/
/**
 * @addtogroup GDrop
 * @{
 */

#define FM 1                    /**< first marker (on chromosome)             */
#define OM 2                    /**< other than first marker                  */
#define FT 3                    /**< trait is first locus on chromosome       */
#define IT 4                    /**< intermediate tloc - has preceding        */
                                /**< marker/tloc locus & following marker     */
#define ET 5                    /**< tloc with no following markers           */
#define UT 6                    /**< unlinked tloc                            */

                        /*recombMap:  identifies neighbors and  gender-
                                      specific probabilities used in
                                      recombination algorithms.
                                      markers & tlocs are in map order;
                                      unlinked tlocs in numerical order       */
struct recombMap {
    int no;
    int recombTyp;
    int iA;
    int iB;
    double probA[2];
    double probB[2];
                 };
typedef struct recombMap RMAP;
typedef RMAP * RMAP_PTR;
GLOBAL RMAP_PTR PRecombMap;

                  /*trait_pars_node:  For each trait, parameters for computing
                                      and printing additive and residual
                                      variances for quantitative traits.
                                      Then for each trait, parameters for
                                      computing and printing incomplete
                                      penetrances and genotype means at each
                                      tloc for discrete or quantitative traits.
                                      In ascending order by trait number.     */
struct trait_pars_node {
    int t_no;
    int num_tlocs;
    struct tloc_pars_node * p_tloc_pars;
    double addi_var;
    double addi_term_found;
    double addi_term;
    double resi_var;
    double resi_term;
                       };
typedef struct trait_pars_node TRAITPARS;
typedef TRAITPARS * TRAITPARS_PTR;
GLOBAL TRAITPARS_PTR P_trait_pars;

struct tloc_pars_node {
    int tl_no;
    int num_penetrances;
    double * penetrances;
    int num_means;
    double * means;
    int wid;
    int dec;
    int ixG;
    int ixPh;
                      };
typedef struct tloc_pars_node TLOCPARS;
typedef TLOCPARS * TLOCPARS_PTR;

                        /*traitPars:  Used in proc_drop: For each trait,
                                      parameters for computing and writing out
                                      the quantitative trait.
                                      In ascending order by trait number.     */
struct traitPars {
    int no;
    double means[3];
    double addi_var;
    double resi_var;
    double addi_term_found;
    double addi_term;
    double resi_term;
    int wid;
    int dec;
    int tlno;
    int ixG;
    int ixPh;
                 };
typedef struct traitPars TPARS;
typedef TPARS * TPARS_PTR;
GLOBAL TPARS_PTR PTraitPars;

                        /*tlocOut:  parameters for writing out the tloc.
                                    tloc nos are in numerical order           */
struct tlocOut {
    int no;
    int ixG;
                };
typedef struct tlocOut TLOUT;
typedef TLOUT * TLOUT_PTR;
GLOBAL TLOUT_PTR PTLocOut;

                        /*alleleFreq:  allele frequencies for each locus      */
struct alleleFreq{
    int nAlleles;                       /*no. of alleles at this locus        */
    double * pCumFreq;                  /*ptr to list of cumulative freqs     */
                 };
typedef struct alleleFreq ALLELE;
typedef ALLELE * ALLELE_PTR;
GLOBAL ALLELE_PTR PAlleleFreq;

GLOBAL int WidAlle;                     /*no. characters for printing alleles */


/** the following variables are used by Genedrop and Autozyg programs  **/

GLOBAL int NLocWndw;            /* window size (no. loci)               */
                                /* for ibddrop and lm_auto              */
GLOBAL int NWndw;               /* no. of windows for the chromosome    */


/** @} */

/*** these data structures are used by Autozyg and Lodscore programs  ***/

/**
 * @addtogroup autozyg
 * @{
 */

#ifdef  DEFINE_DATA

GLOBAL int PedDatTyp = UNDEF;           /* INHERITANCE for pair of      */
                                        /*     inheritance indicators   */
                                        /* GENOTYPIC for genotype code  */
                                        /* DISCRETE for phenotype       */
                                        /* DISCR_LIAB for discrete plus */
                                        /*     liability class          */
                                        /* DISCR_COVAR for discrete plus*/
                                        /*     covariate (age)          */
                                        /* QUANTITATIVE for continuous  */

GLOBAL int Lodscores_at_markers = Y;    /* Historically Lodscore programs     */
                                        /* provided lodscores at all markers  */
                                        /* without a specification from the   */
                                        /* user to do this.  This variable    */
                                        /* was added so that this behavior    */
                                        /* could be readily suppressed.       */

GLOBAL int TraitTyp = UNDEF;
                                        /* GENOTYPIC for genotype code  */
                                        /* DISCRETE for phenotypic      */
                                        /* QUANTITATIVE for continuous  */

GLOBAL int NAforMarkers = FALSE;        /* For discrete traits with at least  */
                                        /* one zero incomplete penetrance or  */
                                        /* for genotypic traits, lodscores at */
                                        /* the markers should not be printed. */
                                        /* If this variable is set to TRUE,   */
                                        /* replace these lodscores with "NA". */

GLOBAL int SmplSetup = UNDEF;           /* which sample setup to use    */
GLOBAL int NSeqImp = UNDEF;             /* number of samples for        */
                                        /* sequential imputation        */
GLOBAL int Scan = UNDEF;                /* sample by scan (T) or single */
                                        /* step(F)                      */
GLOBAL double LSamplProb = 0.2;         /* locus sampler probability    */
GLOBAL int WindWdth = 6;                /* width of test position wndow */
GLOBAL int SeqImpIt = UNDEF;            /* how often to propose seq imp */
GLOBAL int ScorIt = UNDEF;              /* score ECLLD and recomb counts*/
                                        /* every ScorIt steps or scans  */
GLOBAL int NBrnIt = UNDEF;              /* number of burnin iterations  */
GLOBAL int NPseudoPrIt = UNDEF;         /* number of preliminary iter   */
GLOBAL int NBtch = 20;                  /* number of batches MC vari est*/
GLOBAL int BatchSize = UNDEF;           /* Number of sampling iterations to   */
/*                                         be included within each batch.     */
GLOBAL int NIt  = UNDEF;                /* number of step or scan       */
                                        /* iterations (main)            */
GLOBAL int ChkIt  = UNDEF;              /* how often to report iter no. */
GLOBAL int FilIt = UNDEF;               /* how often to write outpt file*/
GLOBAL int OutRB = UNDEF;               /* flag for writting rao-black  */
                                        /* estimates to a file          */
GLOBAL int InPseudo = N;                /* flag for pseudo-priors input */

GLOBAL int MMSampler = UNDEF;           /* Indicator of whether the multiple  */
                                        /* meiosis sampler (when set to Y) or */
                                        /* the LM sampler (when set to N) is  */
                                        /* to be used as the MCMC sampler.    */
GLOBAL int MaxNMexact = 8;              /* Maximum number of meioses allowed  */
                                        /* for exact lod scores calculation.  */
GLOBAL int PrintFGLs = N;               /* Founder genome labels (FGL's) are  */
                                        /* to be output if this is set to Y,  */
                                        /* but not when it is set to N.       */
GLOBAL int PrintMIs = N;                /* Meiosis indicators (MI's) are to   */
                                        /* be output when this is set to Y,   */
                                        /* but not when it is set to N.       */

                                        /* parameters used by lm_map:         */
GLOBAL int MapEstMod = UNDEF;           /* map estimation model               */
                                        /* (with or without mistyping         */
GLOBAL int MaximAlgo = UNDEF;           /* algorithm for maximization         */
                                        /* (hybrid SA/MCEM, or MCEM           */
GLOBAL int RedoEst = 1;                 /* number of indep MC runs            */
GLOBAL int NChiSqIt = 500;              /* number of LRT iterations           */
GLOBAL int NHybAscentIt = 500;          /* number of hybrid ascent iterations */
GLOBAL int NHybCurvIt = 5000;           /* number of hybrid curvature iter    */
GLOBAL int NSAGradIt = 100;             /* number of SA gradient iterations   */
GLOBAL double SA_bound = .005;          /* % change test for SA iterates      */
GLOBAL double GenoErrRat = 0;           /* genotyping error rate              */
GLOBAL double EM_bound = .2;            /* % change test for EM iterates      */
GLOBAL double RF_bound = .01;           /* min recomb fract estimate          */
GLOBAL int MapTyp = UNDEF;              /* output map type                    */
                                        /* (avg, gender-spec, or both)        */

                              /* Trait test positions:                        */
                              /* Total number of positions on chromosome for  */
GLOBAL int NTPos = UNDEF;     /* lod scoring.  This is the number derived     */
                              /* from the parameter statements.               */
GLOBAL int NTPos1 = UNDEF;    /* This is the old definition of NTPos.         */

GLOBAL int McmcMarkersOnly = FALSE; /* When this is set to FALSE (the default */
/*                                     setting), sample at both the tloc(s)   */
/*                                     and at the markers.  When it is set to */
/*                                     TRUE, sample at the marker loci only.  */
GLOBAL int GlobalMcmc;
GLOBAL int ChkMarksConsistent = FALSE;  /* When this is set to FALSE (the     */
                                        /* default setting), the current run  */
                                        /* is not explicitly being run as a   */
                                        /* check for Mendelian inconsistency. */
                                        /* When it is set to TRUE, then this  */
                                        /* run is explicitly to check for any */
                                        /* Mendelian inconsistencies in the   */
                                        /* marker data.  This variable is     */
                                        /* incremented by 1 each time that an */
                                        /* inconsistency is detected.         */

#else
GLOBAL int PedDatTyp;
GLOBAL int Lodscores_at_markers;
GLOBAL int TraitTyp;
GLOBAL int NAforMarkers;
GLOBAL int SmplSetup;
GLOBAL int NSeqImp;
GLOBAL int Scan;
GLOBAL double LSamplProb;
GLOBAL int WindWdth;
GLOBAL int SeqImpIt;
GLOBAL int ScorIt;
GLOBAL int NBrnIt;
GLOBAL int NPseudoPrIt;
GLOBAL int NBtch;
GLOBAL int BatchSize;
GLOBAL int NIt;
GLOBAL int ChkIt;
GLOBAL int FilIt;
GLOBAL int OutRB;
GLOBAL int InPseudo;

GLOBAL int MMSampler;
GLOBAL int MaxNMexact;
GLOBAL int PrintFGLs;
GLOBAL int PrintMIs;

GLOBAL int MapEstMod;
GLOBAL int MaximAlgo;
GLOBAL int RedoEst;
GLOBAL int NChiSqIt;
GLOBAL int NHybAscentIt;
GLOBAL int NHybCurvIt;
GLOBAL int NSAGradIt;
GLOBAL double SA_bound;
GLOBAL double GenoErrRat;
GLOBAL double EM_bound;
GLOBAL double RF_bound;
GLOBAL int MapTyp;
GLOBAL int NTPos;
GLOBAL int NTPos1;
GLOBAL int McmcMarkersOnly;
GLOBAL int GlobalMcmc;
GLOBAL int ChkMarksConsistent;
#endif  /*DEFINE_DATA*/

/* The following definitions are for use by lm_twoqtl, which is a Lodscore    */
/* program.                                                                   */

#ifdef  DEFINE_DATA
GLOBAL int NQtl = UNDEF;
GLOBAL int UseMCSummation = UNDEF;
GLOBAL int MCRealizations = UNDEF;
GLOBAL int GxGInteraction = FALSE;
GLOBAL int MultiplierForNull = 10;
GLOBAL double **GxGMeans = NULL;
#else
GLOBAL int NQtl;              /* Number of quantitative trait loci in model.  */
GLOBAL int UseMCSummation;    /* 0 -> use exact summation for the trait,      */
                              /* 1 -> use MC summation for the trait.         */
GLOBAL int MCRealizations;    /* Number of MC realizations for the trait.     */
GLOBAL int GxGInteraction;    /* 1 -> gene-gene interaction, 0 otherwise.     */
GLOBAL int MultiplierForNull; /* Number of evaluations of the null hypothesis */
                              /* for each evaluation of the alternative       */
                              /* hypothesis.                                  */
GLOBAL double **GxGMeans;     /* Genotypic means for the interacting tlocs.   */
#endif  /*DEFINE_DATA*/

#ifdef  DEFINE_DATA                                  /* default values */
GLOBAL int IBDStats     = UNDEF;    /* Compute IBD statistics          */
GLOBAL int LikRatStats  = UNDEF;    /* Compute likelihood-ratio stats  */
/* IBD statistics measures:                                            */
GLOBAL int SpairsOn     = UNDEF;    /* S pairs                         */
GLOBAL int RobdomOn     = UNDEF;    /* S robdom                        */
GLOBAL int SAffUnaffOn  = UNDEF;    /* S affect unaff                  */
GLOBAL int LambdaOn     = UNDEF;    /* S lambda                        */
GLOBAL int MaxFGLOn     = UNDEF;    /* max FGL                         */
/* IBD statistics tests:                                               */
GLOBAL int NormalEnable = UNDEF;    /* normal                          */
GLOBAL int PermutEnable = UNDEF;    /* permutation                     */
/* No. IBD permutations                                                */
GLOBAL int NPermutIt = 999;

/* likelihood-ratio statistics measures:                               */
GLOBAL int DeltaOn      = UNDEF;    /* delta                           */
GLOBAL int Lambda_pOn   = UNDEF;    /* lambda-p                        */
/* likelihood-ratio statistics tests:                                  */
GLOBAL int ChiSqEnable  = UNDEF;    /* chi-squared                     */
/* likelihood-ratio lambda-p model gridpoints                          */
GLOBAL int NumOfSegL    = 6;        /* in lambda_a, lambda_u interval  */
GLOBAL int NumOfSegP    = 9;        /* in interval for p               */
/* likelihood-ratio lambda-p model grid spacing                        */
GLOBAL double IntervalSegL;         /* for lambda_a, lambda_u interval */
GLOBAL double IntervalSegP;         /* for interval for p              */

#else
GLOBAL int IBDStats;
GLOBAL int LikRatStats;
GLOBAL int SpairsOn;
GLOBAL int RobdomOn;
GLOBAL int SAffUnaffOn;
GLOBAL int LambdaOn;
GLOBAL int MaxFGLOn;
GLOBAL int NormalEnable;
GLOBAL int PermutEnable;
GLOBAL int NPermutIt;
GLOBAL int DeltaOn;
GLOBAL int Lambda_pOn;
GLOBAL int ChiSqEnable;
GLOBAL int NumOfSegL;
GLOBAL int NumOfSegP;
GLOBAL double IntervalSegL;
GLOBAL double IntervalSegP;
#endif  /*DEFINE_DATA*/

GLOBAL int NLocPos;           /* no. of marker plus test trait positions      */

                              /* LocPos:  description of each marker          */
                              /*            and trait test position           */
struct locus_pos {
    int locus_typ;            /* is this: selected marker                     */
                              /*          non-selectd marker test pos         */
                              /*          tloc test position                  */
    int intv_no;              /* interval no., 0 ... n                        */
                              /*     0 : before marker 1,                     */
                              /*     i : marker i,                            */
                              /*         between marker i, i+1, if test pos   */
                              /*     n : marker n,                            */
                              /*         or following marker n, if test pos   */
    int no;                   /* marker or trait no. (no. from maps)          */
    double pos[2];            /* sex-specific position of marker or tloc      */
    double m1dist[2];         /* sex-specific distance to marker 1, in cM     */
    double rf[2];             /* sex-specific recomb fract with preced marker */
                              /* or marker 1, for interval "0"                */
    double rfnxt[2];          /* sex-specific recomb fract with next marker   */
                     };
typedef struct locus_pos LOCPOS;
typedef LOCPOS * LOCPOS_PTR;
GLOBAL LOCPOS_PTR LocPos;

typedef struct joint_locus_pos JNTLOCPOS;
typedef JNTLOCPOS * JNTLOCPOS_PTR;
GLOBAL JNTLOCPOS_PTR JntLocPos;

GLOBAL char ** MarkNam;       /* Name of each marker                          */
GLOBAL char ** TLocNam;       /* Name of each tloc                            */
GLOBAL char ** TraitNam;      /* Name of each trait                           */
GLOBAL double ** MarkPos;     /* For ea gender, position of ea marker (cM)    */
GLOBAL double TraitPos[2];    /* for ea gender, position of the trait (cM)    */

GLOBAL int LocNamLen;         /* No. chars in longest marker or trait name    */

GLOBAL int * NTPosInt;        /* no. of trait test positions for ea interval. */

                              /* alleles:                                     */
                              /* no. of alleles at each locus                 */
                              /* NAll[0]...NAll[Nloci-1]                      */
GLOBAL int * NAll;            /* ordered by name (chromo[][])                 */

                              /* allele frequencies at each locus             */
                              /* Qq[0][1]....Qq[0][NAll[0]]                   */
GLOBAL double **Qq;           /*          ...Qq[NLoci-1][NAll[NLoci-1]]       */

                              /* marker data:                                 */
GLOBAL int NObs;              /* no. individuals observed                     */
                              /* (autozyg, for marker data,                   */
                              /* (polyEM, for selected traits)                */
GLOBAL char ** ObsNam;        /* names of observed individuals                */
GLOBAL int  *  ObsIx;         /* indices of observed individuals              */
GLOBAL int *** ObsMark;       /* for ea observed individual, for ea           */
                              /* marker, a pair of alleles                    */

                              /* proband gametes and pattern labels:    */
GLOBAL int MxGam;             /* the largest number of gametes in a set */

                              /* proband gametes:                       */
GLOBAL int NScCmp;            /* greatest component no. with proband    */
                              /* gamete sets;  may be reset to no.      */
                              /* components read if smaller             */
GLOBAL int NScCmpAlloc;       /* greatest comp. no. with proband gamete */
                              /* sets (for freeing)                     */
GLOBAL int * NScSet;          /* for each component, the number of sets */
                              /* of proband gametes                     */
GLOBAL int ** ScSetNo;        /* for each component for each set,       */
                              /* its set no. (need not be contiguous)   */
GLOBAL int ** NPrGam;         /* for each component for each set,       */
                              /* number of probands                     */
GLOBAL char **** ProbNam;     /* for each component for each set,       */
                              /* the list of names of individuals       */
                              /* corresponding to the proband gametes   */
                              /* to be scored                           */
GLOBAL int *** ProbIx;        /* the list of indices to the individuals */
GLOBAL int *** ProbGam;       /* the list of gametes corresponding to   */
                              /* the list of individuals                */

                              /* IBD pattern labels:                    */
GLOBAL int ** NPtLab;         /* for each component and each set,       */
                              /* number of IBD pattern labels           */
GLOBAL int *** PatLab;        /* for each component and each set,       */
                              /* the list of IBD pattern labels         */

                              /* lm_bayes preliminary step:             */
GLOBAL double * PseudoPrior;  /* the pseudo-priors (input or calculated)*/
                              /* for each position                      */

/* Marker recombination fractions.  The second dimension is NLoci numbers     */
/* long.  The first subscript is for sex: 0 = female; 1 = male.  The second   */
/* subscript is for marker: Marek[][0] = 0.5 (not used); Marek[][1] =         */
/* recombination fraction between marker 1 and marker 2; and in general       */
/* Marek[][n] = recombination fraction between marker n and marker n+1.       */
/* The last entry is Marek[][NLoci] = 0.5.                                    */

GLOBAL double ** Marek;

/* MarSel was added in Apr 2010 (EAT).  It is set in proc_auto and proc_lods  */
/* to pass on the original indices of all the selected markers.               */

GLOBAL int  * MarSel;

/* Trial positions for linked trait.                                          */

GLOBAL int Loc0;
GLOBAL double Rec0[2];        /* and gender-specific recomb fractions   */

GLOBAL int * IntVal;          /* [NTPos+2] test positions for trait     */
GLOBAL double ** RecVal;      /* [2][NTPos+2] sex-specifc recomb fracts */
                              /* (extra position on ea end with recomb  */
                              /* fraction of .5                         */

/**** the following variables are used for graphics displays        *****/
/*
 * Autozyg/lm_auto graphics uses OpenGl and freeglut
 * The GLDisp/ library contains graphics functions to call these
 */

GLOBAL int GDispOpt;           /* DEFLT_DISP, SEGR_DISP, LABL_DISP      */
GLOBAL int GDispIt;            /* how often to display                  */
GLOBAL int GDispNIt;           /* current iteration                     */

GLOBAL int GDispNamLen;        /* how many chars are needed for name    */
GLOBAL int NGDispCmp;          /* no. of ped components to display      */
GLOBAL int * NGDispNams;       /* no. individuals to display for cmpnt  */
GLOBAL char *** GDispNam;      /* names of individusal                  */
GLOBAL int ** GDispIx;         /* indices to individuals                */

GLOBAL int NGDispLoc;          /* no. of loci to display                */
GLOBAL int * GDispLoc;         /* loci to display                       */


/***** The following variables are for Morgan programmers.              */

GLOBAL int DebugMain;
GLOBAL int DebugCMF;
GLOBAL int DebugGLDisp;
GLOBAL int DebugMarkers;
GLOBAL int DebugNghds;
GLOBAL int DebugPars;
GLOBAL int DebugPedchk;
GLOBAL int DebugPeel;
GLOBAL int DebugQuant;
GLOBAL int DebugRans;
GLOBAL int DebugSample;
GLOBAL int DebugStuff;
GLOBAL int DebugTwoqtl;

/* The following variable is being added so that at least temporarily I will  */
/* be able to continue using the debug_level options in lm_twoqtl.  It is     */
/* conceivable that we might want to add a general debug_level option to the  */
/* preceding binary debug variables, so that Morgan subroutines might want to */
/* distinguish different levels of debugging.                                 */

#ifdef  DEFINE_DATA
GLOBAL int DebugLevel = 0;
#else
GLOBAL int DebugLevel;
#endif  /*DEFINE_DATA*/

/** @} */

#endif  /*GLOBS_H*/
