/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "traits.h"

/* includes markers.h and others included therein
 * see traits.h for specs and purpose of routines in this file.
 *                                                            EAT March 2002
 */

#include "lsample.h"

int print_traits (int nt, int *loc_fl)
{
   char  tt_name[13] = "";
   int   tt_name_len;
   int   nm = 0;
   int   i1;
   int   n;

   n = 4;
   if (LocNamLen > n)
      n = LocNamLen;

   if (PrintLevel >= 5)
   {
      printf ("\n Genetic variables in the analysis:\n");
      printf (" %8s    %-*s   %4s\n\n", "index", n, "name", "type");
   }

   for (i1 = 0; i1 < nt; ++i1)
   {
      if (PrintLevel >= 5 && traits[i1].trait_type != UNDEF)
         printf (" %8d    %-*s", i1, n, traits[i1].self);

      switch (traits[i1].trait_type)
      {
          case MARKER_GENO:
             tt_name_len = sprintf (tt_name, "geno-marker");
             traits[i1].locus = loc_fl[traits[i1].locus];
             ++nm;
             break;
          case GENOTYPIC:
             tt_name_len = sprintf (tt_name, "genotypic");
             break;
          case DISCRETE:
             tt_name_len = sprintf (tt_name, "discrete");
             break;
          case ORDINAL:
             tt_name_len = sprintf (tt_name, "ordinal");
             break;
          case QUANTITATIVE:
             tt_name_len = sprintf (tt_name, "quantitative");
             break;
          case INHERITANCE:
             tt_name_len = sprintf (tt_name, "inheritance");
             break;
          default:
             break;
      }

      if (PrintLevel >= 5 && traits[i1].trait_type != UNDEF)
         printf ("   %*s\n", tt_name_len, tt_name);
   }

   if (PrintLevel >= 5)
   {
      printf ("\n\n");
      fflush (stdout);
   }

   return (nm);
}

void recess2 (double *vv, const struct Trait *trait,
              const Ind * id, const Loclist * ll, double lam)
/* will become a penetrance routine for a discrete trait */
{
   return;
}

/* ****************** JUNK STUFF ABOUT LINKED LIST *******************
   Indlist *child_ptr;

   for (j = i + 1; j < limits[1] + 1 && parent[j][2] == parent[i][2]; ++j)
   {
   child_ptr->next = (Indlist *) malloc (sizeof (Indlist));
   alloc_chk (WHERE, child_ptr->next);
   child_ptr = child_ptr->next;
   child_ptr->current = &nghd[parent[j][0]];
   }
   }
   child_ptr->next = NULL;
   ***************************************************** */

void default_traits (int nt)
/* set up default names and specs for traits */
{
   int   i, m = 0;
   struct Trait *trait;

   for (i = 0; i < nt; i++)
   {
      trait = traits + i;
      trait->index = i;
      trait->locus = i;
      if (i == 0)                       /* trait locus */
      {
         if (TraitTyp == UNDEF)         /* null locus */
         {
            trait->trait_type = UNDEF;
            strncpy (trait->self, "", MAXNAMLEN + 1);
         }
         else if (TraitTyp == GENOTYPIC)
         {
            trait->trait_type = GENOTYPIC;
            strncpy (trait->self, "trt-geno", MAXNAMLEN + 1);
         }
         else if (TraitTyp == DISCRETE) /* phenotypic data */
         {
            trait->trait_type = DISCRETE;
            trait->penetrance = recess2;
            strncpy (trait->self, "trt-pheno", MAXNAMLEN + 1);
         }
         else if (TraitTyp == QUANTITATIVE) /* continuous data */
         {
            trait->trait_type = QUANTITATIVE;
            strncpy (trait->self, "trt-quant", MAXNAMLEN + 1);
         }
         else if (TraitTyp == INHERITANCE) /* seg indicators */
         {
            trait->trait_type = INHERITANCE;
            strncpy (trait->self, "", MAXNAMLEN + 1);
         }
      }
      else
      {                                 /* marker locus */
         trait->trait_type = MARKER_GENO;
         strncpy (trait->self, MarkNam[m], MAXNAMLEN + 1);
         ++m;
      }
      if (LocNamLen < strlen (trait->self))
         LocNamLen = strlen (trait->self);
   }

   return;
}

void flag_loci (int nt, int nloc, int *loc_fl)
/*  Store indices in an array of integer locus flags to each selected marker  */
/*  locus.  The first index will be set to 1.  The remaining indices increase */
/*  by 1 up to the number of selected markers.                                */
{
   int   count_of_markers;
   int   i1;

   /* Initialize all of the integer locus flags to -1.                        */
   for (i1 = 0; i1 <= nloc; ++i1)
      loc_fl[i1] = -1;

   /* Change the integer locus flags corresponding to marker loci to +1.      */
   for (i1 = 0; i1 < nt; ++i1)
      if (traits[i1].trait_type == MARKER_GENO)
         loc_fl[traits[i1].locus] = 1;

   /* Now convert the integer locus flags corresponding to marker loci into   */
   /* indices representing which selected marker the locus flag refers to.    */
   /* The first index is set to 1.  The remaining indices increase by 1 up to */
   /* the number of selected markers.  This subroutine is written the way it  */
   /* is so that if there are one or more trait loci amongst the marker loci, */
   /* the indexing of marker loci will still be correct.                      */

   count_of_markers = 0;
   for (i1 = 0; i1 <= nloc; ++i1)
      if (loc_fl[i1] > -1)
      {
         ++count_of_markers;
         loc_fl[i1] = count_of_markers;
      }

   return;
}

int set_traits (int nl, int *loc_fl)
{
   int   nt;                            /* Number of traits.                  */
   /* This number of traits is currently set to the number of loci plus 1.    */
   nt = nl + 1;

/* Allocate trait structures.                                                 */

   traits = malloc (sizeof (struct Trait) * nt);
   alloc_chk (WHERE, traits);

/* Set traits for the default case.                                           */
   default_traits (nt);

/* Flag marker loci that are defined by the traits.                           */
   flag_loci (nt, nl, loc_fl);

/* Return the number of traits.                                               */
   return (nt);
}

void free_traits (void)
{
   if (traits)
   {
      free (traits);
      traits = NULL;
   }
   return;
}

int set_eval_points (int nloc, CTP_PTR p_ctp)
{
   LOCPOS_PTR p_p;

   double *rec_left[2], *rec_rght[2];
   int   sex;
   int   p1;
   int   x1;

   if (DebugMarkers)
   {
      printf (" Table of evaluation points:\n");
      printf ("   loc_left  male-rec    fem-rec");
      printf ("   loc_right  male-rec    fem-rec\n");
   }

   rec_left[0] = p_ctp->rec_left[0];
   rec_left[1] = p_ctp->rec_left[1];
   rec_rght[0] = p_ctp->rec_rght[0];
   rec_rght[1] = p_ctp->rec_rght[1];

   p_ctp->loc_left[0] = 0;
   p_ctp->loc_rght[0] = 0;
   for (sex = FEM; sex <= MAL; ++sex)
   {
      rec_left[sex][0] = 0.0;
      rec_rght[sex][0] = 0.0;
   }

   for (p1 = 1, p_p = LocPos, x1 = 1; p1 <= NLocPos; ++p1, ++p_p)
   {
      if (p_p->locus_typ == MARK_LOC)   /* Test for selected marker position. */
      {

/* Only use locus position recombination fractions for selected markers when  */
/* the traditional evaluation at markers is being done.  If a "map test tlocs */
/* ... at markers ..." statement is in effect, don't include locus position   */
/* data for selected markers.                                                 */
         if (Lodscores_at_markers)
         {
            p_ctp->loc_left[x1] = p_p->intv_no;
            p_ctp->loc_rght[x1] = p_p->intv_no;
            for (sex = FEM; sex <= MAL; ++sex)
            {
               rec_left[sex][x1] = 0.0;
               rec_rght[sex][x1] = 0.0;
            }
            ++x1;
         }
      }

      else                              /* If not a selected marker position: */
      {
         if (p_p->intv_no == 0)         /* Test position before first marker? */
         {
            p_ctp->loc_left[x1] = -1;
            p_ctp->loc_rght[x1] = 1;
         }
         else if (p_p->intv_no < NMarks) /* Test position within interval?    */
         {
            p_ctp->loc_left[x1] = p_p->intv_no;
            p_ctp->loc_rght[x1] = p_p->intv_no + 1;
         }
         else                           /* Test position after last marker?   */
         {
            p_ctp->loc_left[x1] = p_p->intv_no;
            p_ctp->loc_rght[x1] = -1;
         }

         for (sex = FEM; sex <= MAL; ++sex) /* Sets recombination fractions.  */
         {
            rec_left[sex][x1] = p_p->rf[sex];
            rec_rght[sex][x1] = p_p->rfnxt[sex];
         }
         ++x1;
      }
   }

   if (DebugMarkers)
   {
      printf ("       %d     %f   %f       %d      %f   %f\n",
              p_ctp->loc_left[p1], rec_left[1][p1], rec_left[0][p1],
              p_ctp->loc_rght[p1], rec_rght[1][p1], rec_rght[0][p1]);
      fflush (stdout);
   }

   return (x1);
}
