/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "mulmsample.h"

/* cmp = 0: Consider all components simultaneously.                           */
/* cmp > 0: Indicates which component to consider.                            */
/* mul = 0: M-sampler; otherwise, MM-sampler.                                 */
/* ntp =    Number of test positions.                                         */

void do_mcmc_sampling (int cmp, int mul, int ntp, int *p_scored_iterations,
                       double **prob, double *mcmc_lod, double *mcmc_sd,
                       CTP_PTR p_ctp, RLD_PTR p_rld, MSD_PTR p_msd,
                       FILE * oScorF)
{
   double *unused = NULL;

   int   bc;                            /* Current batch indicator.           */

   int   iteration;
   int   p1;

   prntStatusBMC ();                    /* Print status for beginning MCMC.   */

   /* Setup variables for LM-multiple sampling: LPT Apr 2006.                 */
   setup_sampling_cmp (cmp, mul, (NLoci - 1));

   /* ................  BURN-IN  .......................................      */

   /* Now do burn-in iterations.                                              */

   printf (" Now starting %d iterations of burn-in\n", NBrnIt);
   for (iteration = 1; iteration <= NBrnIt; ++iteration)
      sampling_cmp (iteration, cmp, NTLocs, mul, (NLoci - 1), nFGL, prob);
   printf ("\n");
   fflush (stdout);

   /* ................  COMPUTE USING MCMC  ............................      */
   bc = 0;                              /* Set current batch indicator to 0.  */

   *p_scored_iterations = 0;            /*Initialize scored iterations count. */

   /* Now do the regular MCMC sampling, and score the results.                */

   printf (" Now starting %d iterations of MCMC\n", NIt);
   fflush (stdout);

   for (iteration = 1; iteration <= NIt; ++iteration)
   {
      /* Update xx1, xx0.                                                     */
      sampling_cmp (iteration, cmp, NTLocs, mul, (NLoci - 1), nFGL, prob);

      if (DebugMain)
      {
         fprintf (stderr, "\nAt iteration %d,\n", iteration);
         dbgXXGG ();
      }

      if (!(iteration % ScorIt))
      {
         update_counts ("", p_msd, cmp, 1, prob, unused);

         /* Score rb_likelihoods.                                             */

         if (bc < NBtch)
            do_rblik_cmp (cmp, ntp, bc, p_ctp, p_rld);

         /* Every FilIt scored iterations, the current test positions log     */
         /* likelihoods need to be output to the interim scores file.         */

         ++(*p_scored_iterations);

         /* Tests to see whether or not to output to scores file.             */
         if (FilIt && !(*p_scored_iterations % FilIt))
            /*Modified 5/2010 to output log-likelihoods of scored iterations. */
         {
            fprintf (oScorF, "\n log-likelihoods: iter %d, cmpt %d\n",
                     iteration, cmp);
            for (p1 = 1; p1 < ntp; ++p1)
               fprintf (oScorF, " %f", p_rld->cmpt_lik[p1]);
            fprintf (oScorF, "\n");
         }

         /* Start new batch if current batch is complete.                     */

         if (!(*p_scored_iterations % BatchSize))
            ++bc;
      }
   }

   /* Print status that we have finished with the MCMC.                       */
   printf ("\n\n");
   prntStatusFMC ();
   fflush (stdout);

   free_sampling_cmp ();

   return;
}

void set_batch_size (void)
{
   int   scored_iterations;             /* Number of scored iterations.       */

   scored_iterations = compute_scored_iterations ();
   BatchSize = scored_iterations / NBtch;

/* Check that the number of batches divides the number of scored iterations.  */
   if (BatchSize * NBtch != scored_iterations)
      give_warn (ERL_bt1, NBtch);

/* Make sure that BatchSize is a positive number.                             */
   if (BatchSize <= 0)
      give_up (ERL_bt2, NBtch);

   printf (" %d realizations will be scored, at scoring interval %d\n",
           scored_iterations, ScorIt);
   printf (" Scored realizations will be batched in %d sets of size %d\n",
           NBtch, BatchSize);

   return;
}

int compute_scored_iterations (void)
{
   int   scored_iterations;             /* Total number of iterations to be   */
   /*                                      used for lod score estimation.     */
   /*                                      For example, if 1000 iterations    */
   /*                                      are scored every 20, then          */
   /*                                      scored_iterations = 50.            */

   scored_iterations = NIt / ScorIt;
   if (scored_iterations * ScorIt != NIt)
      give_warn (ERR_err,
                 "Scoring interval does not divide realizations count");

   return scored_iterations;
}

CTP_PTR alloc_ctp (int npos)
{
/* double rec_left[2][NLocPos + 1], double rec_right[2][NLocPos + 1],         */
/* int    loc_left[NLocPos + 1],    int    loc_right[NLocPos + 1]             */

   CTP_PTR p_ctp;
   int   x1;

   p_ctp = malloc (sizeof (CTP));
   alloc_chk (WHERE, p_ctp);

   p_ctp->rec_left = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, p_ctp->rec_left);
   p_ctp->rec_rght = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, p_ctp->rec_rght);

   for (x1 = 0; x1 < 2; ++x1)
   {
      p_ctp->rec_left[x1] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_left[x1]);
      p_ctp->rec_rght[x1] = malloc (npos * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_rght[x1]);
   }

   p_ctp->loc_left = malloc (npos * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_left);
   p_ctp->loc_rght = malloc (npos * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_rght);

   return p_ctp;
}

/* Reallocate the four arrays in the cur_test_pos (current test position)     */
/* structure passed to this subroutine.  They need to be reduced in size      */
/* from (NLocPos + 1), as they were originally allocated, to just ntp, as was */
/* determined by the set_eval_points subroutine.  The resulting arrays will   */
/* then be:                                                                   */
/*                                                                            */
/*    double rec_left[2][ntp], double rec_right[2][ntp],                      */
/*    int    loc_left[ntp],    int    loc_right[ntp]                          */

void realloc_ctp (int ntp, CTP_PTR p_ctp)
{
   int   x1;

   for (x1 = 0; x1 < 2; ++x1)
   {
      p_ctp->rec_left[x1] =
         realloc (p_ctp->rec_left[x1], ntp * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_left[x1]);
      p_ctp->rec_rght[x1] =
         realloc (p_ctp->rec_rght[x1], ntp * sizeof (double));
      alloc_chk (WHERE, p_ctp->rec_rght[x1]);
   }

   p_ctp->loc_left = realloc (p_ctp->loc_left, ntp * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_left);
   p_ctp->loc_rght = realloc (p_ctp->loc_rght, ntp * sizeof (int));
   alloc_chk (WHERE, p_ctp->loc_rght);

   return;
}

void free_ctp (CTP_PTR p_ctp)
{
   free_2d ((void *) &p_ctp->rec_left);
   free_2d ((void *) &p_ctp->rec_rght);
   free_1d ((void *) &p_ctp->loc_left);
   free_1d ((void *) &p_ctp->loc_rght);
   free (p_ctp);

   return;
}

/******************************************************************************/
/*                                                                            */
/* The alloc_rld subroutine is called to allocate and initialize to 0.0's     */
/* whichever of the 4 double precision floating point arrays selected by the  */
/* caller from among the arrays used to hold the cumulative rb_likelihoods    */
/* within batches and by test position, along with their corresponding monte  */
/* carlo standard deviations.                                                 */
/*                                                                            */
/* The first argument to this subroutine is a null-terminated string.  This   */
/* string is used to indicate which of the arrays is to be allocated and      */
/* initialized by this subroutine.  The following table includes the type of  */
/* data in each array, the name of the array, the dimensions of the array and */
/* the options code letter included as a character in the first argument to   */
/* tell the subroutine to allocate and initialize the array.                  */
/*                                                                            */
/* array data type   array name       dimensions       options code letter    */
/*                                                                            */
/*     double        cmpt_lik    [NTPos + NMarks + 1]          'C'            */
/*     double        rb_prob     [NTPos + NMarks + 1]          'P'            */
/*     double        rb_prbm     [NBtch][NTPos + NMarks + 1]   'B'            */
/*     double        rb_sd       [NTPos + NMarks + 1]          'S'            */
/*                                                                            */
/******************************************************************************/

RLD_PTR alloc_rld (const char *options, int npos, int nb)
{
   RLD_PTR p_rld;

   p_rld = malloc (sizeof (RLD));
   alloc_chk (WHERE, p_rld);
   p_rld->base_lik = 0.0;

   if (strchr (options, 'C'))
      p_rld->cmpt_lik = alloc_dbl1d (npos);
   else
      p_rld->cmpt_lik = NULL;

   if (strchr (options, 'P'))
      p_rld->rb_prob = alloc_dbl1d (npos);
   else
      p_rld->rb_prob = NULL;

   if (strchr (options, 'B'))
      p_rld->rb_prbm = alloc_dbl2d (nb, npos);
   else
      p_rld->rb_prbm = NULL;

   if (strchr (options, 'S'))
      p_rld->rb_sd = alloc_dbl1d (npos);
   else
      p_rld->rb_sd = NULL;

   return p_rld;
}

void free_rld (RLD_PTR p_rld)
{
   free_1d ((void *) &p_rld->cmpt_lik);
   free_1d ((void *) &p_rld->rb_prob);
   free_2d ((void *) &p_rld->rb_prbm);
   free_1d ((void *) &p_rld->rb_sd);
   free (p_rld);

   return;
}

MSD_PTR alloc_msd (void)
{
   MSD_PTR p_msd;

   p_msd = malloc (sizeof (MSD));
   alloc_chk (WHERE, p_msd);

   p_msd->recounts = alloc_int3d (n_compnts, NMarks, 6);
   p_msd->tot_log_prob_seg = alloc_dbl2d (n_compnts, NMarks);

   /* The tot_log_prob_pen matrix needs to have an extra column allocated to  */
   /* store data for the last marker.                                         */
   p_msd->tot_log_prob_pen = alloc_dbl2d (n_compnts, NMarks + 1);

   p_msd->temp_log_prob_seg = alloc_dbl2d (n_compnts, NMarks);

   return p_msd;
}

void free_msd (MSD_PTR p_msd)
{
   free_3d ((void *) &p_msd->recounts);
   free_2d ((void *) &p_msd->tot_log_prob_seg);
   free_2d ((void *) &p_msd->tot_log_prob_pen);
   free_2d ((void *) &p_msd->temp_log_prob_seg);
   free (p_msd);

   return;
}

/******************************************************************************/
/*                                                                            */
/* The zero_mcmc_stats subroutine is called to set the entries in 3 of the    */
/* arrays located in the MCMC sampling statistics data structure to zeroes.   */
/* As of when this subroutine was added, it does not appear to be necessary   */
/* to zero the entries in the temp_log_prob_seg array.                        */
/*                                                                            */
/******************************************************************************/

void zero_mcmc_stats (MSD_PTR p_msd)
{
   memset (p_msd->recounts[0][0], 0, n_compnts * NMarks * 6 * sizeof (int));
   memset (p_msd->tot_log_prob_seg[0], 0.0,
           n_compnts * NMarks * sizeof (double));
   memset (p_msd->tot_log_prob_pen[0], 0.0,
           n_compnts * (NMarks + 1) * sizeof (double));

   return;
}

/******************************************************************************/
/*                                                                            */
/* The update_counts subroutine may be called to update the MCMC sampling     */
/* statistical data counts structure, which is the second argument to this    */
/* subroutine.  The third argument, comp_no, is either a 0 or the number of   */
/* the component which has just been sampled.  If this argument is 0, then    */
/* statistics for all components are to be updated simultaneously.  Otherwise */
/* only statistics for the just sampled component are to be updated.  The     */
/* fourth argument, first_locus, is either a zero or a one.  If it is a zero, */
/* the zero'th locus is included in the update.  If it is a one, the zero'th  */
/* locus is not included in the update.                                       */
/*                                                                            */
/* If the options string contains a 'C', then the subroutine also updates the */
/* specified component's complete data log likelihood, comp_cdll[comp_no].    */
/* Note that this subroutine is written so that comp_cdll is not referred to  */
/* unless options contains a 'C'.                                             */
/*                                                                            */
/******************************************************************************/

void update_counts (const char *options, MSD_PTR p_msd, int comp_no,
                    int first_locus, double **prob, double *comp_cdll)
{
   int   c1;                            /* Current component subscript.       */

   if (comp_no > 0)
   {
      /* Do the statistics update for the component which was sampled.        */
      update_mcmc_stats (options, p_msd, comp_cdll, first_locus, prob,
                         comp_no - 1, 0);
   }
   else
   {
      /* Do the statistics update for all components simultaneously.          */
      for (c1 = 0; c1 < n_compnts; ++c1)
         update_mcmc_stats (options, p_msd, comp_cdll, first_locus, prob,
                            c1, c1);
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Update the recombination count and the total log-probabilities for the     */
/* specified component number using the passed component index to subscript   */
/* within the mcmc_stats_data structure.  The first_locus argument is either  */
/* a zero or a one.  If it is a zero, the zero'th locus is included in the    */
/* update.  If it is a one, the zero'th locus is not included in the update.  */
/*                                                                            */
/* If the options string contains a 'C', then the subroutine also updates the */
/* specified component's complete data log likelihood, comp_cdll[comp_no].    */
/* Note that this subroutine is written so that comp_cdll is not referred to  */
/* unless options contains a 'C'.                                             */
/*                                                                            */
/******************************************************************************/

void update_mcmc_stats (const char *options, MSD_PTR p_msd, double *comp_cdll,
                        int first_locus, double **prob, int comp_no,
                        int comp_ndx)
{
   unsigned update_cdll;
   int   p1;
   int   p2;

   update_cdll = strchr (options, 'C') != NULL;

   full_seg_prob_cmp (NMarks, comp_no, p_msd->recounts[comp_ndx],
                      p_msd->temp_log_prob_seg[comp_ndx]);
   if (update_cdll)
      comp_cdll[comp_ndx] = 0.0;

   p2 = (first_locus == 1) ? max (NTLocs, 1) : 0;

   for (p1 = first_locus; p1 < NMarks; ++p1, ++p2)
   {
      p_msd->tot_log_prob_pen[comp_ndx][p1] += log (prob[comp_no][p2]);
      p_msd->tot_log_prob_seg[comp_ndx][p1] +=
         p_msd->temp_log_prob_seg[comp_ndx][p1];

      if (update_cdll)
         comp_cdll[comp_ndx] += (log (prob[comp_no][p2]) +
                                 p_msd->temp_log_prob_seg[comp_ndx][p1]);
   }

   p2 = NLoci - 1;

   /* It is because of the following statement that tot_log_prob_pen needs    */
   /* NMarks + 1 columns, whereas tot_log_prob_seg only needs NMarks columns. */

   p_msd->tot_log_prob_pen[comp_ndx][NMarks] += log (prob[comp_no][p2]);

   if (update_cdll)
      comp_cdll[comp_ndx] += log (prob[comp_no][p2]);

   return;
}

/******************************************************************************/
/*                                                                            */
/* Since both the mcmc_lods subroutine and the lm_twoqtl program can provide  */
/* MCMC and lod score results, this subroutine was implemented.  It consists  */
/* of precisely the code shared by the two.                                   */
/*                                                                            */
/******************************************************************************/

void mcmc_lods_results (RLD_PTR p_rld, MSD_PTR p_msd, CTP_PTR p_ctp, int ntp,
                        int cmp, int scored_iterations, double *lod, double *sd)
{
   int   bc;                            /* Current batch indicator.           */
   int   p1;                            /* Current test position subscript.   */
   int   c1;                            /* Current component number.          */

   /* Compute likelihood estimates.                                           */

   for (p1 = 1; p1 < ntp; ++p1)
   {
      p_rld->rb_prob[p1] = p_rld->rb_prob[p1] / (NBtch * BatchSize);
      for (bc = 0; bc < NBtch; ++bc)
         p_rld->rb_prbm[bc][p1] = p_rld->rb_prbm[bc][p1] / BatchSize;
   }

   /* Batch-means standard devation estimates.                                */
   /* Careful with component indexing in arrays!                              */

   for (p1 = 1; p1 < ntp; ++p1)
   {
      for (bc = 0; bc < NBtch; ++bc)
         p_rld->rb_sd[p1] += (p_rld->rb_prbm[bc][p1] - p_rld->rb_prob[p1]) *
            (p_rld->rb_prbm[bc][p1] - p_rld->rb_prob[p1]);
      p_rld->rb_sd[p1] = sqrt (p_rld->rb_sd[p1] / ((NBtch - 1) * NBtch));
   }

   /* Output results for this simulation/estimation set.                      */
   /* Moved down so likelihoods can also be by component: EAT July 2004.      */

   if (cmp > 0)
   {
      pr_cmpnthead (cmp - 1);
      pr_comp_mcmc_stats (p_msd, 1, scored_iterations, cmp, 0);
   }
   else
   {
      for (c1 = 1; c1 <= n_compnts; ++c1)
      {
         pr_cmpnthead (c1 - 1);
         pr_comp_mcmc_stats (p_msd, 1, scored_iterations, c1, c1 - 1);
      }
   }

   /* Now compute estimated lodscores at each test position along with their  */
   /* Monte Carlo standard deviations.                                        */

   for (p1 = 1; p1 < ntp; ++p1)
      if ((p_ctp->loc_left[p1] != p_ctp->loc_rght[p1]) || !NAforMarkers)
      {
         lod[p1] = log10 (p_rld->rb_prob[p1]);
         sd[p1] = p_rld->rb_sd[p1] / (p_rld->rb_prob[p1] * log (10.0));
      }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Call the 3 regularly called subroutines for printing the MCMC sampling     */
/* statistics for the specified component number using the passed component   */
/* index to subscript within the mcmc_stats_data structure.                   */
/*                                                                            */
/* The first_row argument passed to this subroutine contains the subscript of */
/* the first row in the tot_log_prob_seg and the tot_log_prob_pen matrices in */
/* which there is any useful data stored.  It should be either a 0 or a 1.    */
/* This should be a 0 if there was a tloc used in the MCMC sampling.  If no   */
/* tloc was used in sampling, then this argument should be a 1.               */
/*                                                                            */
/******************************************************************************/

void pr_comp_mcmc_stats (MSD_PTR p_msd, int first_row, int scored_iterations,
                         int comp_no, int comp_ndx)
{
   if (PrintLevel >= 4)
   {
      pr_log_prob_seg (first_row, NMarks, -cmplist[comp_no].nmeioses,
                       p_msd->tot_log_prob_seg[comp_ndx], scored_iterations);
      pr_log_prob_pen (first_row, NMarks, p_msd->tot_log_prob_pen[comp_ndx],
                       scored_iterations);
      pr_reco_counts (first_row, NMarks, p_msd->recounts[comp_ndx]);
   }

   return;
}
