/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "msample.h"

void setupSampling (void)
{
   /* set up data structures for locus and meiosis sampling. 
    * fncs:  alloc_xxgg(), allocate_loci(), set_quant_trait(), alloc_msmpl()
    * vars:  int xx0[NMarks+1][PedSize], xx1[NMarks+1][PedSize] 
    *        int gg0[NMarks+1][PedSize], gg1[NMarks+1][PedSize]
    *        Locus loci[NMarks+1].*  
    *        int new_segs[NMarks+1],
    *        double pp2[NMarks][2], pcf [NMarks][2]
    * dirs:  Sample
    */

   setSampling ();
   if (!IS_MARKERDROP)
      setMSampling ();

   /* initialize seeds for rans().
    * vars:  pSeedC, pSeedT   
    * dirs:  Rans
    */
   if (IS_LM_IBDTESTS && PermutEnable)
      setSeeds (PERM);
   else
      setSeeds (SAMPL);

   return;
}

/**
 * Calls functions to set up initial values for inheritance indicators 
 * and Founder Gene Labels prior to burn-in and the main MCMC iterations.
 * mm is 0 for sampling the trait, mm is 1 for markers only, no trait.
 */
double **setupInherit (int mm)
{
   double **pr;

   /* initialize maternal and paternal inheritance indicators 
    * and founder genome labels (FGLs). 
    * fncs:  prntStatusSMC(), set_seg_mode(), set_lsmpl(), do_seq_imp(),
    *        prntStatusIMC(), set_FGL()
    * vars:  xx0[][], xx1[][], gg0[][], gg1[][]
    * dirs:  Sample, Stuff
    */
   initInherit (mm);

   /* set up peeling for founder genome labels.
    * fncs:  alloc_seg_pen(), alloc_gen_pen()
    * vars:  first[9*ngenes[j]], pp[2][ngenes[j]], obslist[2*NMarks][n], etc.
    * dirs:  Sample
    */

   /*  FGL peeling enabled for trait locus, even if not in MCMC: EAT 4/2010 
    */
   setFGLpeel (0);

   /* set up array for storing seg_pen results.
    * fncs:  seg_pen2()
    * vars:  prob[nCompnt][NMarks+1]
    * dirs:  Sample
    */

   pr = initPenProb (mm);

   return pr;
}
